/*
 * JTella SearchExample
 *
 * Copyright (c) 2000 Ken McCrary, All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies.
 *
 * KEN MCCRARY MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. KEN MCCRARY
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT
 * OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
import com.kenmccrary.jtella.NetworkConnection;
import com.kenmccrary.jtella.ConnectionListener;
import com.kenmccrary.jtella.ConnectionEvent;
import com.kenmccrary.jtella.SearchReplyMessage;
import com.kenmccrary.jtella.MessageReceiverAdapter;
import com.kenmccrary.jtella.SearchSession;


/**
 *  An example class for showing how to execute search queries on the
 *  GNUTella network using JTella. The application expects two command
 *  line parameters, the host name, and the port the servant is listening on
 *
 */
public class SearchExample implements ConnectionListener
{
  private NetworkConnection conn;

  /**
   *  Constructs the example given a started network connection
   *
   */
  public SearchExample(NetworkConnection networkConnection)
  {
    this.conn = networkConnection;
  }


  /**
   *  Invoked when a connection change occurs
   *
   *  @param event connection event
   */
  public void connectionChange(ConnectionEvent event)
  {
    int connectionCount = event.getConnectionInformation().getCount();

    if ( 0 == connectionCount ) 
    {
      System.out.println("DISCONNECTED");
    }

    if ( 1 == connectionCount ) 
    {
      System.out.println("CONNECTED");
    }

    // When active connections are available run the example
    if ( 2 == connectionCount) 
    {
      System.out.println("Sending search request");

      SearchSession search1 = conn.createSearchSession(new TestReceiver());
      search1.search("elvis", 100, 0);
      SearchSession search2 = conn.createSearchSession(new TestReceiver());
      search2.search("madonna", 100, 0);
      SearchSession search3 = conn.createSearchSession(new TestReceiver());
      search3.search("beavis", 100, 0);
      SearchSession search4 = conn.createSearchSession(new TestReceiver());
      search4.search("santana", 100, 0);
    }
  }
  
  /**
   *  Entry point for the search example application, host and port number are
   *  expected arguments
   *
   */
  public static void main(String[] args)
  {
    System.out.println("<--- JTella Search Example running --->");


    if (args.length != 2)
    {
      System.out.println("Usage: SearchExample host port");
      System.exit(1);
    }

    try
    {
      //-------------------------------------------------------------
      // Start a network connection and listen for succesful connection
      //-------------------------------------------------------------
      NetworkConnection c = new NetworkConnection(args[0],
                                  Integer.decode(args[1]).intValue());
      c.addConnectionListener(new SearchExample(c));
      c.start();

    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

  }

  /**
   *  Test class for receiving search responses
   *
   */
  static class TestReceiver extends MessageReceiverAdapter
  {

    /**
     *  For the example, just print the search response info to the console
     *
     */
    public void receiveSearchReply(SearchReplyMessage searchReply)
    {
      System.out.println("****Test receiver session received reply****");
      System.out.println("Port: " + searchReply.getPort());
      System.out.println("IP Address:" + searchReply.getIPAddress());
      System.out.println("FileCount:" + searchReply.getFileCount());

      for (int i = 0 ; i <  searchReply.getFileCount(); i++) 
      {
        SearchReplyMessage.FileRecord fileRecord = searchReply.getFileRecord(i);
        System.out.println("FileRecord: " +
                            i +
                            ", name: " +
                            fileRecord.getName() +
                            ", size: " +
                            fileRecord.getSize() +
                            ", index: " + 
                           fileRecord.getIndex());
      }

      System.out.println("****END Search session received reply****");
  }

  }
}
