/*
 * Copyright (C) 2000 Ken McCrary
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. (www.gnu.org)
 */
package com.kenmccrary.jtella;

import java.util.Vector;
import java.util.Enumeration;

import com.kenmccrary.jtella.util.Log;

/**
 *  A cache of the known hosts on the network
 *
 */
// TODO complete
public class HostCache
{
  private Vector hosts;


  /**
   * Constructs an empty HostCach
   *
   */
  public HostCache()
  {
    hosts = new Vector();
  }
  
  /**
   *   Adds a host to the cache
   *
   *   @param ipAddress internet address
   *   @param port port 
   */
  void addHost(String ipAddress, int port)
  {
     Log.getLog().logInformation("Adding host: " + ipAddress + ":" + port);
     hosts.addElement(new Host(ipAddress, port));
  }

  /**
   *  Remove a host from the cache, probably because its not responding
   *
   */
  void removeHost(Host host)
  {
    hosts.removeElement(host);

  }

  /**
   *  Query how many hosts are cached
   *  
   *  @return number of hosts
   */
  int size()
  {
    return hosts.size();
  }

  /**
   *  Get the next host available
   *
   *  @return host or null if none available
   */
  Host nextHost()
  {
    if ( size() == 0 ) 
    {
      return null;
    }

    return (Host)(getHosts().nextElement());
  }

  /**
   *  Get an enumeration of the hosts cached
   *
   */
  Enumeration getHosts()
  {
    return hosts.elements();
  }

  /**
   *  Represents a gnutella host
   *
   */
  static class Host
  {
     String ipAddress;
     int port;

     /**
      *  Construct a host record
      *
      *  @param ipAddress host address
      */
     Host(String ipAddress, int port)
     {
       this.ipAddress = ipAddress;
       this.port = port;
     }

     /**
      *  Returns the address
      *
      */
     String getIPAddress()
     {
       return ipAddress;
     }

     /**
      * Returns the port
      *
      */
     int getPort()
     {
       return port;
     }

     /**
      *  Equals comparison
      *
      *
      */
     public boolean equals(Object obj)
     {
       if ( !(obj instanceof Host) ) 
       {
         return false;
       }


       Host rhs = (Host)obj;

       return getIPAddress().equals(rhs.getIPAddress());
     }

     /**
      *  Use the IP address for the hashcode
      */
     public int hashCode()
     {
       return getIPAddress().hashCode();
     }
   }

}
