/*
 * Copyright (C) 2000 Ken McCrary
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. (www.gnu.org)
 */
package com.kenmccrary.jtella;

import com.kenmccrary.jtella.util.Log;

/**
 *  Construct the appropriate <code>Message</code>
 *  subclass
 *
 */
class MessageFactory
{
  /**
   *  Location of the function type in the header
   *
   */
  final static int typeLocation = 16;

  /**
   *  Factory method providing messages
   *
   */
  static Message createMessage(byte[] rawMessage)
  {
    short type = rawMessage[typeLocation];
    Message message = null;

    switch ( type ) 
    {
      case Message.PING:
      {
        Log.getLog().logInformation("MessageFactory created a ping message");
        message = new PingMessage(rawMessage);
        break;
      }

      case Message.PONG:
      {
        Log.getLog().logInformation("MessageFactory created a pong message");
        message = new PongMessage(rawMessage);
        break;
      }

      case Message.QUERY:
      {
        Log.getLog().logInformation("MessageFactory created a query message");
        message = new SearchMessage(rawMessage);
        break;
      } 

      case Message.QUERYREPLY:
      {
        Log.getLog().logInformation("MessageFactory created a query reply message");
        message = new SearchReplyMessage(rawMessage);
        break;
      }

      case Message.PUSH:
      {
        Log.getLog().logInformation("MessageFactory created a push message");
        message = new PushMessage(rawMessage);
        break;
      }

      default:
      {
        Log.getLog().logError("Message factory can't create unknown message type: " + type);
        break;
      }
    }

    return message;
  }
}
