/*
 * Copyright (C) 2000 Ken McCrary
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. (www.gnu.org)
 */
package com.kenmccrary.jtella;

import java.util.HashMap;

/**
 *   Contains information for routing originated messages
 *
 *
 */
class OriginateTable 
{
  private HashMap hashMap;

  OriginateTable()
  {
    hashMap = new HashMap();
  }
  
  /**
   *  Puts a GUID to MessageReceiver mapping in the table
   *
   *
   */
  synchronized void put(GUID guid, MessageReceiver receiver)
  {
    hashMap.put(guid, receiver);
  }

  /**
   *  Get a message receiver for a GUID
   *
   *
   */
  synchronized MessageReceiver get(GUID guid)
  {
    return (MessageReceiver)hashMap.get(guid);
  }

  /**
   *  Check if <code>MessageReceiver</code> exists for guid
   *  This is equivalent to checking if we sent a message
   *
   */
  boolean containsGUID(GUID guid)
  {
    return hashMap.containsKey(guid);
  }
}
