/*
 * Copyright (C) 2000 Ken McCrary
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. (www.gnu.org)
 */
package com.kenmccrary.jtella;

import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;

import com.kenmccrary.jtella.util.Log;

/**
 *  Contains history information on Messages used for routing
 *  Maps GUIDs to the connection they arrived on
 *
 */
class RouteTable
{
  private HashMap hashMap;
  private int maxSize;

  /**
   *  Construct the RouteTable, indicating maximum size
   *
   *  @param maxSize maximum number of records to maintain
   */
  RouteTable(int maxSize)
  {
    hashMap = new HashMap();
    this.maxSize = maxSize;
  }

  /**
   *  Puts a GUID to Connection mapping in the table
   *
   *
   */
  synchronized void put(GUID guid, BaseConnection connection)
  {
    if ( hashMap.size() > maxSize ) 
    {
      // constrain the history size        
      Set set = hashMap.entrySet();
      Iterator iterator = set.iterator();

      while ( iterator.hasNext() ) 
      {
        // move to end
      }

      Log.getLog().logInformation("Trimming route table");
      iterator.remove();
    }

    hashMap.put(guid, connection);
  }

  /**
   *  Retrieves a Connection for a GUID
   *
   */
  synchronized BaseConnection get(GUID guid)
  {
      return (BaseConnection)hashMap.get(guid);
  }

  /**
   *  Check if Connection history exists for guid
   *
   */
  boolean containsGUID(GUID guid)
  {
    return hashMap.containsKey(guid);
  }
}

