/*
 * Copyright (C) 2000 Ken McCrary
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. (www.gnu.org)
 */
package com.kenmccrary.jtella;

import java.util.Vector;
import java.util.Enumeration;

/**
 *  Search Reply message, response to a search request
 *
 *
 */
public class SearchReplyMessage extends Message
{
  private FileRecord[] fileRecords;

  /**
   *  Construct a SearchReply message
   *
   *
   */
  SearchReplyMessage()
  {
    super(Message.QUERYREPLY);
  }

  /**
   *  Construct a search reply message from data read
   *  read from network
   *
   *  @param rawMessage
   */
  SearchReplyMessage(byte[] rawMessage)
  {
    super(rawMessage);
  }

  /**
   *  Query the umber of files found for the search
   *
   */
  public int getFileCount()
  {
    return payload[0];
  }


  /**
   *  Query the port for this search reply
   *
   *  @return port
   */
  public int getPort()
  {
    int port = 0;
    int byte1 = 0;
    int byte2 = 0;
    byte1 |= payload[2];
    byte2 |= payload[2];

    port |= byte1;
    port |= (byte2 << 8);

    return port;
  }


  /**
   *  Query the IP address for this pong message
   *  result is an IP address in the form of
   *  "206.26.48.100".
   *
   *  @return IP address
   */
  public String getIPAddress()
  {
    StringBuffer ipBuffer = new StringBuffer();

    ipBuffer.append(Integer.toString(payload[3])).
             append(".").
             append(Integer.toString(payload[4])).
             append(".").
             append(Integer.toString(payload[5])).
             append(".").
             append(Integer.toString(payload[6]));

    return ipBuffer.toString();
  }

  /**
   *  Get information about the files found
   *
   */
  public FileRecord getFileRecord(int index)
  {
    if ( null == fileRecords) 
    {
      fileRecords = new FileRecord[getFileCount()];
      int payloadIndex = 11;

      for (int i = 0; i < getFileCount(); i ++ ) 
      {
        // TODO calculate size/index

        // Read the index
        int index1 = payload[payloadIndex++];
        int index2 = payload[payloadIndex++];
        int index3 = payload[payloadIndex++];
        int index4 = payload[payloadIndex++];

        int fileIndex = 0;
        fileIndex |= index1;
        fileIndex |= index2 << 8;
        fileIndex |= index3 << 16;
        fileIndex |= index4 << 24;

        // Read the size
        int size1 = payload[payloadIndex++];
        int size2 = payload[payloadIndex++];
        int size3 = payload[payloadIndex++];
        int size4 = payload[payloadIndex++];
        
        long fileSize = 0L;
        fileSize |= size1;
        fileSize |= size2 << 8;
        fileSize |= size3 << 16;
        fileSize |= size4 << 24;
        
        // Read the file terminated by double null
        // temp
        int nullCount = 0;
        Vector stringData = new Vector();
        while ( 2 != nullCount ) 
        {
          
          byte b = (byte)payload[payloadIndex++];
          stringData.addElement(new Byte(b));
          if ( 0 == b ) 
          {
            nullCount++;
          }
          else
          {
            nullCount = 0;
          }


        }

        byte[] stringBytes = new byte[stringData.size()];

        for(int z = 0; z < stringBytes.length; z++)
        {
          stringBytes[z] = ((Byte)stringData.get(z)).byteValue();
        }

        fileRecords[i] = new FileRecord(fileIndex,
                                        fileSize,
                                        new String(stringBytes));

      }
    }

    return fileRecords[index];
  }


  // TODO host speed accessor

  /**
   *  Information about the files returned in this request
   *
   */
  public class FileRecord
  {
     int index;
     long size;
     String fileName;

     FileRecord(int index, long size, String fileName)
     {
       this.index = index;
       this.size = size;
       this.fileName = fileName;
     }

     public int getIndex()
     {
       return index;
     }

     public long getSize()
     {
       return size;
     }

     public String getName()
     {
       return fileName;
     }
  }
}
