/*
 * Copyright (C) 2000 Ken McCrary
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. (www.gnu.org)
 */
package com.kenmccrary.jtella;

import com.kenmccrary.jtella.util.Log;

/**
 *  Represents a Session on a Connection which 
 *  can be used for GNUTella searchs
 *  
 */
public class SearchSession
{
  private ConnectionControl connectionControl;
  private MessageReceiver receiver;

  SearchSession(ConnectionControl connectionControl,
                MessageReceiver receiver)
  {
    this.connectionControl = connectionControl;
    this.receiver = receiver;
  }

  /**
   *  Starts a search on the GNUTella network
   *
   */
  public void search(String query, int maxResults, int minSpeed)
  {
    // Create a search message
    SearchMessage searchMessage = new SearchMessage(query, minSpeed);   

    // Get a connection and send it

    BaseConnection connection = connectionControl.getNextConnection();
    try
    {
      connection.sendAndReceive(searchMessage, receiver);
    }
    catch (Exception e)
    {
      Log.getLog().log(e);
    }
  }

}
