/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: gnutella-client
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-02-10, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2001-02-10, Marcus Wolschon<Suran@gmx.net>)
  *       - download
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.gnutella;

  //JDK
import java.rmi.server.UnicastRemoteObject;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.io.Serializable;
import java.lang.IllegalStateException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.registry.LocateRegistry;
import java.util.LinkedList;
//import java.net.URL;
  //JINI
import net.jini.core.event.RemoteEventListener;
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.Lease;
import net.jini.admin.Administrable;
  //project
import jaugment2.jini.simpleService.simpleAdministrator;
import jaugment2.texteditor.TextEditorProxyIF.ActionsEntry;
import jaugment2.helpers.Logger;

//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


/**
 * A simple web-browser using Sun's HotJavaBean
 */
public class GnutellaProxy extends UnicastRemoteObject implements Administrable, Serializable, Remote , GnutellaProxyIF
{
 private GnutellaService service;


  GnutellaProxy(GnutellaService service ) throws RemoteException
  {
   try
     {
      this.service = service;
     }
   catch(Exception e)
     {
      Logger.err.println("Exception in GnutellaProxy::GnutellaProxy() has the Message: "+e.getMessage());
      e.printStackTrace(Logger.err);
     }
  }


  /*public void pause() throws RemoteException, IllegalArgumentException, IllegalStateException
  {
   service.pause();
  } */




 /**
   * Returns an object that implements our administration-interfaces.
   *
   * @throws java.rmi.RemoteException
   * @see JoinAdmin
   */
  public Object getAdmin() throws RemoteException
  {
   return(new simpleAdministrator(service));
  }


  //-------------- Client-functions ------------------------------

  /**
   * return entries for all mime-actions found
   */
  public ActionsEntry[] getActionsEntries() throws RemoteException
  {
   return(service.getActionsEntries());
  };


  /** Subscribe for buffer-eddings, removals and changes
    */
  public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException
  {
   return(service.addListener(listener, handback, registerfor));
  }

  public void connect(String host, int port) throws RemoteException
  {
   service.connect(host, port);
  };

  public void addSearch(String searchfor, int maxResults, int minSpeed) throws RemoteException
  {
   service.addSearch(searchfor, maxResults, minSpeed);
  };


  public void download(String url) throws RemoteException
  {
   service.download(url, null);
  }
  public void download(String url, String to) throws RemoteException
  {
   service.download(url, to);
  }

}