/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: gnutella-client
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1h (2001-02-10, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-02-24, Marcus Wolschon<Suran@gmx.net>)
  *      - stop, restart,... super.* to properly fire events
  * -------------------------------------------------------------------
  * TODO:
  *       - test it some more
  *       - give it a real UI (monitor searches, download, administer, monitor up/down-loads)
  *       - serve files (http-server)
  *       - allow getting files (http-download)
  */
package jaugment2.gnutella;

  //JDK
import java.rmi.RemoteException;
import java.rmi.Remote;
import java.rmi.MarshalledObject;
import java.util.Properties;
import java.util.LinkedList;
import java.util.Hashtable;
import java.util.Set;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Enumeration;
import java.lang.IllegalArgumentException;
import java.lang.IllegalStateException;
import java.lang.StringBuffer;
import java.lang.Math;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.BufferedInputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.File;
import java.util.StringTokenizer;
import java.util.Collection;
import java.net.UnknownHostException;
import java.net.URL;
  //JINI
import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.lease.landlord.Landlord.RenewResults;
import com.sun.jini.lease.landlord.LocalLandlord;
import com.sun.jini.lease.landlord.LandlordLeaseFactory;
import com.sun.jini.lease.landlord.LandlordUtil;
import com.sun.jini.lease.landlord.LandlordLease;
import com.sun.jini.lease.landlord.LeasedResource;
import com.sun.jini.lease.landlord.LandlordLease.Factory;
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
//import net.jini.core.event.EventRegistration;
import net.jini.core.discovery.LookupLocator;
import net.jini.core.entry.Entry;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.factory.JComponentFactory;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
  //JTella
import com.kenmccrary.jtella.Utilities;
import com.kenmccrary.jtella.NetworkConnection;
import com.kenmccrary.jtella.ConnectionListener;
import com.kenmccrary.jtella.ConnectionEvent;
import com.kenmccrary.jtella.MessageReceiver;
import com.kenmccrary.jtella.SearchSession;
import com.kenmccrary.jtella.SearchReplyMessage;
import com.kenmccrary.jtella.SearchMessage;
import com.kenmccrary.jtella.SearchMonitorSession;
import com.kenmccrary.jtella.HostCache;
  //project
import jaugment2.jini.simpleService.*;
import jaugment2.jini.LookupEventListener;
import jaugment2.jini.LookupEventEvent;
import jaugment2.texteditor.TextEditorProxyIF.ActionsEntry;
import jaugment2.gnutella.GnutellaProxyIF.GnutellaEvent;
import jaugment2.gnutella.GnutellaProxyIF.*;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;
import jaugment2.helpers.Logger;


//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper

/**
 * A simple web-browser using Sun's HotJavaBean
 */
public class GnutellaService extends simpleService implements Remote , LookupEventListener
{

 private static final boolean debugMessages=true;
 private String defaultDownloadDir = "/tmp";


  //-------------------- simple Service

 /** restart this service once it has been stopped
   * @see stop
   */
 public void restart() throws Exception
 {
  super.restart();
 }

 /** stop this service. If exit is false all references to this
   * service are still valid but it has deregistered everywhere.
   * (this includes the RMI-activation-daemon)
   * @see restart
   * @see destroy
   * @param exit stop running
   */
 public void stop(boolean exit) throws Exception
 {
  super.stop(exit);
 }

 /** stop this service, destroy any persistent storage and exit
   */
 public void destroy() throws RemoteException
 {
  super.destroy();
 }


  //-------------------- Constructor
  public GnutellaService() throws RemoteException
  {
   this(null);
  };


  GnutellaProxy proxy;

  public GnutellaService( LookupLocator firstLocator) throws RemoteException
  {
//   landLord = new SimpleLandlord(simpleLeaselistener);
   try
     {
      persistentState.setProperty("groups", "public, Suran");
      persistentState.setProperty("locators", "jini://localhost, jini://127.0.0.1");


      defaultDownloadDir = System.getProperty("jaugment2.gnutella.GnutellaService.defaultDownloadDir");
      if(defaultDownloadDir==null)
        defaultDownloadDir="/tmp";

      proxy = new GnutellaProxy(this);
      landLord = new SimpleLandlord(simpleLeaselistener);

      loadMe();

      Entry[] attrSets = (Entry[])persistentState.get("attr");

      init(proxy, attrSets, this, firstLocator);


     }
   catch(Exception e)
     {
      Logger.err.println(e.getMessage());
      e.printStackTrace(Logger.err);
     }

  }

  private void loadMe()
  {
      String filename = System.getProperty("jaugment2.gnutella.GnutellaService.persistentStateFilename");
      if(filename==null)
        {
         filename = "GnutellaServiceState.dat";
         System.setProperty("jaugment2.gnutella.GnutellaService.persistentStateFilename", filename);
        }

      persistentState.setProperty("persistentStateFilename", filename);


   load();


   if(persistentState.get("attr")==null)
     {
      Entry[] attrSets = new Entry[]
                    {
                     new net.jini.lookup.entry.Name("Gnutella-client"),
                     new net.jini.lookup.entry.Comment("A simple gnutella-client using jTella"),
                     new net.jini.lookup.entry.ServiceInfo("Gnutella-client", "Marcus Wolschon <Suran@gmx.net>", "Marcus Wolschon <Suran@gmx.net>", "0.0.1", "standart", "00000"),
//                     WebbrowserSwingRepresentation.getUIDescriptor(),
                     GnutellaSwingRepresentation2.getUIDescriptor(),
//                     new BrowseTo_MimeDataHandlerEntry()
//                     PasswordServiceTextRepresentation.getUIDescriptor()
                    };
      persistentState.put("attr", attrSets);
     }

   save();

  }



  //------------------------------------


  private Hashtable serviceID2ActionsEntry = new Hashtable();

  public void serviceFoundEvent(LookupEventEvent e)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] serviceFoundEvent");
   if(e.item != null)
   if(e.item.service != null)
     {
        // integrate mime-actions
      ActionsEntry entry = new ActionsEntry(e.item);
      if(entry.getMimeDataHandlerEntryNumber()>0)
        {
         if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] found a ["+entry.getProxy().getClass().toString()+"] with "+entry.getMimeDataHandlerEntryNumber()+" possible actions e.item.serviceID="+e.item.serviceID);
         if(serviceID2ActionsEntry.put(e.item.serviceID, entry) != null)
             if(debugMessages)Logger.err.println("ERROR: [GnutellaService] serviceID="+e.item.serviceID+" is used by multiple services!!!");
         //TODO: inform our listeners
        }
     }

  }

  public void serviceGoneEvent(LookupEventEvent e)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] serviceGoneEvent");
   //TODO
   if(e.item != null)
   if(e.item.service != null)
     {
         //remove action-entries
      ActionsEntry entry = (ActionsEntry)serviceID2ActionsEntry.remove(e.item.serviceID);
      if(entry != null)
        {
         Logger.debug.println("DEBUG: [GnutellaService] lost a ["+entry.getProxy().getClass().toString()+"] with "+entry.getMimeDataHandlerEntryNumber()+" possible actions");
         //TODO: inform our listeners
        }
     }
  }

  public void serviceChangedEvent(LookupEventEvent e)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] serviceGoneEvent");
   if(e.item != null)
   if(e.item.service != null)
     {
      ActionsEntry entry = (ActionsEntry)serviceID2ActionsEntry.get(e.item.serviceID);
      if(entry != null)
        {
         entry.reinit(e.item);
         if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] changed a ["+entry.getProxy().getClass().toString()+"] to "+entry.getMimeDataHandlerEntryNumber()+" possible actions");
         if(entry.getMimeDataHandlerEntryNumber()<1)
           {
            serviceID2ActionsEntry.remove(e.item.serviceID);
           }
         //TODO: inform our listeners
        }
     }
  }



  /**
   * return entries for all mime-actions found
   */
  public ActionsEntry[] getActionsEntries() throws RemoteException
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] getActionsEntries() serviceID2ActionsEntry.size()="+serviceID2ActionsEntry.size());
   Collection entriesc  = serviceID2ActionsEntry.values();
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] getActionsEntries() entriesc.size()="+entriesc.size());
   return((ActionsEntry[])entriesc.toArray(new ActionsEntry[0]));
  }


  //------------------------------- events -------------------------------

  /**
   * Registrations of BufferListeners
   */
  private LinkedList listenerRegistrations[] = {null, new LinkedList(), new LinkedList()};

  /** Subscribe for buffer-eddings, removals and changes
    */
  public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException
  {
   if(listener==null)
     return(null);

   try
     {
      EventRegistration reg = new EventRegistration(listener, handback, 0xFF);
      Lease   l = landLord.grantLease(0, reg, 2*60*10);

/*      if((registerfor & WebbrowserEvent.EVID_SETURL) != 0)
        {listenerRegistrations[WebbrowserEvent.EVID_SETURL].add(reg);}

      if((registerfor & WebbrowserEvent.EVID_SETPAGE) != 0)
        {listenerRegistrations[WebbrowserEvent.EVID_SETPAGE].add(reg);}*/

      return(l);
     }
   catch(Exception ex)
     {
      ex.printStackTrace();
      return(null);
     }
  }



  /**
   * Helper-function to inform all our listeners
   */
  private void fireEvent(int type, String data)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService:fireEvent] I have "+listenerRegistrations[type].size()+" registrations");
   for(int i=0; i<listenerRegistrations[type].size(); i++)
     {
      try
        {                     //TODO: do this asyncronously
         EventRegistration entry = (EventRegistration)listenerRegistrations[type].get(i);
         if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService:fireEvent] delivering event type="+type+" subscribedFor="+entry.subscribedFor+" entry.subscribedFor & type="+(entry.subscribedFor & type));
         if( (entry.subscribedFor & type) != 0)
         entry.listener.notify(
                               new GnutellaEvent(getProxy(),
                                                     type,//entry.eventID,
                                                     entry.seq++,
                                                     entry.handback,
                                                     data
                                                    )
                              );
        }
      catch(RemoteException ex)
        {
         Logger.err.println("ERROR: [GnutellaService:fireEvent] could not deliver event");
         ex.printStackTrace(Logger.err);
        }
      catch(UnknownEventException ex)
        {
         Logger.err.println("ERROR: [GnutellaService:fireEvent] could not deliver event - the receiver does not know it");
         ex.printStackTrace(Logger.err);
        }
     }
  }

  //-------------------- leases

  /**
   * Helper-class for out Landlord.
   */
  private LeaseListener simpleLeaselistener = new LeaseListener()
  {
   public void leaseCanceled(int type, Object reg, int newcount)
   {
     /* if((type & WebbrowserEvent.EVID_SETURL) != 0)
        {listenerRegistrations[WebbrowserEvent.EVID_SETURL].remove(reg);}

      if((type & WebbrowserEvent.EVID_SETPAGE) != 0)
        {listenerRegistrations[WebbrowserEvent.EVID_SETPAGE].remove(reg);}*/
   };
  };

  private SimpleLandlord landLord;


  //----------------------- Gnutella

  private LinkedList connections = new LinkedList();
  private LinkedList searches = new LinkedList();

  public void connect(String host, int port)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] shall connect to "+host+":"+port);
   MyConnection mc = new MyConnection(host, port);
   connections.add(mc);
  }

  public void addSearch(String searchfor, int maxResults, int minSpeed)
  {
  if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] shall search for \""+searchfor+"\"");
   MySearch ms = new MySearch(searchfor, maxResults, minSpeed);
   searches.add(ms);//TODO: add this search to all MyConnections with free connections
  }

  private class MyConnection implements ConnectionListener, MessageReceiver
  {
   private NetworkConnection c;
   private String host;
   int port;

   public MyConnection(String host, int port)
   {
    this.host = host;
    this.port = port;
    Thread t = new Thread()
     {
      public void run()
       {
        MyConnection.this.init();
       }
     };
     t.start();
    }

    protected void init()
    {
     try
       {
        this.c = new NetworkConnection(host, port);
        this.c.addConnectionListener(this);
        this.c.start();
        SearchMonitorSession smses = this.c.getSearchMonitorSession(MyConnection.this);
        //TODO: smses
       }
     catch(UnknownHostException uhx)
       {
        Logger.err.println("ERROR: [GnutellaService] host "+host+" is unknown");
        connections.remove(this);
       }
     catch(IOException iox)
       {
        Logger.err.println("ERROR: [GnutellaService] IOException while connecting to host "+host+" says: "+iox.getMessage());
        iox.printStackTrace(Logger.err);
       }
    }

  /**
   *  Implement to receive a message reply
   *
   *  @param searchResult search reply
   */
  public void receiveSearchReply(SearchReplyMessage searchReplyMessage)
  {

  }


  /**
   *   Implement to receive a query message from the network,
   *   this will be called for every query and subsequently should
   *   be implement efficiently
   *
   */
  public void receiveSearch(SearchMessage searchMessage)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] someone is searching for: "+searchMessage.getSearchCriteria()+" (2)");
  }

   /**
    *  Invoked when a connection change occurs
    */
   public void connectionChange(ConnectionEvent event)
   {
    int connectionCount = event.getConnectionInformation().getCount();

    switch(connectionCount)
    {
     case 0:
       {
        if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] disconnected from "+host+":"+port);
        connections.remove(this);
        break;
       }
     case 1:
       {
        if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] connected to "+host+":"+port);
        break;
       }
//     default:
     case 2:
       {
        if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] free connection avaliabler for "+host+":"+port);
        //TODO
        for(int i=0; i<searches.size(); i++)
          {
           MySearch ms = (MySearch)searches.get(i);
           ms.addConnection(c);
          }
        break;
       }
     default:
        break;
    }
   }
  }



  private class MySearch implements MessageReceiver
  {
   private String searchfor;
   private int maxResults;
   private int minSpeed;
   private LinkedList SearchSessions = new LinkedList();
   public MySearch(String searchfor, int maxResults, int minSpeed)
   {
    this.searchfor = searchfor;
    this.maxResults = maxResults;
    this.minSpeed = minSpeed;
   }
   public void addConnection(NetworkConnection c)
   {
    if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] searching for \""+searchfor+"\"");
    SearchSession sses = c.createSearchSession(this);
    sses.search(searchfor, maxResults, minSpeed);
    SearchSessions.add(sses);
   }

  /**
   *   Implement to receive a query message from the network,
   *   this will be called for every query and subsequently should
   *   be implement efficiently
   *
   */
  public void receiveSearch(SearchMessage searchMessage)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] someone is searching for: "+searchMessage.getSearchCriteria());
   //TODO
  }


  /**
   *  Implement to receive a message reply
   *
   *  @param searchResult search reply
   */
  public void receiveSearchReply(SearchReplyMessage searchReplyMessage)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] got an answer for the search of \""+searchfor+"\" from"+searchReplyMessage.getIPAddress()+":"+searchReplyMessage.getPort()+" fileCount="+searchReplyMessage.getFileCount());
   for(int i=0; i<searchReplyMessage.getFileCount(); i++)
     {
      SearchReplyMessage.FileRecord fr = searchReplyMessage.getFileRecord(i);
      if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] file#"+i+"="+fr.getName()+" size="+fr.getSize()+" index="+fr.getIndex());
      fireEvent(GnutellaEvent.EVID_FILEFOUND, "http://"+searchReplyMessage.getIPAddress()+":"+searchReplyMessage.getPort()+"/"+fr.getName()); //TODO: is this a URL?
     }
   //TODO: send answer back to our listeners
  }

  }




  public void download(String url, String to)
  {
   //TODO
   if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] download(String url=\""+url+"\", String to=\""+to+"\")");
   if(to==null)
     to = defaultDownloadDir;

   DownloadThread t = new DownloadThread(url, to);
   t.start();
  }


  private class DownloadThread extends Thread
  {
   private String url;
   private String to;
   public DownloadThread(String url, String to)
   {
    this.url = url;
    this.to  = to;
   }
   public void run()
   {
    try
      {
       URL u = new URL(url);
       File f = new File(to+System.getProperty("file.separator")+u.getFile());
       java.io.InputStream in = u.openStream();
       f.createNewFile();
       OutputStream out = new FileOutputStream(f);
       if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] starting downloading file "+f.getAbsolutePath());
       byte b[] = new byte[Math.max(1024, in.available())];
       int i;
       while( (i = in.read(b)) != -1)
         {
          out.write(b, 0, i);
         }
       in.close();
       out.close();
       if(debugMessages)Logger.debug.println("DEBUG: [GnutellaService] starting downloading file "+f.getAbsolutePath());
      }
    catch(Exception e)
      {
       e.printStackTrace(Logger.err);
      }
   }
  }


}
