/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: gnutella-client
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-02-10, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2001-02-10, Marcus Wolschon<Suran@gmx.net>)
  *       - download
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.gnutella;

  //JDK
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.MarshalledObject;
import java.lang.StringBuffer;
import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.io.IOException;
import java.io.Serializable;
import java.io.StringBufferInputStream;
import java.beans.*;
  //Swing
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
  //JAF
import javax.activation.MimeType;
import javax.activation.DataSource;
import javax.activation.URLDataSource;
  //JINI
import net.jini.core.entry.Entry;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
import net.jini.core.event.EventRegistration;
import net.jini.lease.LeaseRenewalManager;
import net.jini.core.lease.Lease;
import net.jini.core.lookup.ServiceItem;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.ui.MainUI;
import net.jini.lookup.ui.factory.JComponentFactory;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
import net.jini.lookup.ui.attribute.RequiredPackages;
  //project
import jaugment2.UI.swingUI.SwingUIRepresentable;
import jaugment2.helpers.Logger;
import jaugment2.helpers.MimeDataHandlerEntry;
import jaugment2.helpers.FileMimeIdentifier;
import jaugment2.helpers.FileMimeIdentifierEntry;
import jaugment2.texteditor.TextEditorProxyIF.ActionsEntry;
import jaugment2.gnutella.GnutellaProxyIF.GnutellaEvent;

/** A simple web-browser using Sun's HotJavaBean
  */
public class GnutellaSwingRepresentation2  extends JPanel implements Entry, SwingUIRepresentable, RemoteEventListener
{
 protected GnutellaProxyIF proxy;
 private static final boolean debug  = false;
 private LeaseRenewalManager lrm = new LeaseRenewalManager();
  JPanel jPanel_connect;
  JButton jButton_connect;
  JTextField jTextField_hostname;
  JTextField jTextField_port;
  JLabel jLabel1;
  JPanel jPanel_search;
  JButton jButton_search;
  JTextField jTextField_searchstring;
  BorderLayout borderLayout1;
  JPanel jPanel_found;
  JList jList_found;
  BorderLayout borderLayout2;
  JButton jButton_download;



 /**
  * Get a UIDescriptor as needed by the serviceui-draft-standart
  */
 public static UIDescriptor getUIDescriptor()
  {
   try
     {
      return(new UIDescriptor(MainUI.ROLE,
                              JComponentFactory.TOOLKIT,
                              getServiceUIProperties(),
                              new MarshalledObject(new Factory()))
                             );
     }
   catch(IOException e)
     {
      e.printStackTrace(Logger.err);
     }
   return(null);
  }



  /**
   * Gets a serviceui-draft-standart-compactible attribute-set of this representation
   */
  static Set getServiceUIProperties()
  {
   HashSet set = new HashSet();
   set.add("javax.swing.JComponent");
   set.add("javax.swing.JPanel");
   set.add("java.awt.Component");

   HashSet retval = new HashSet();
   retval.add(new UIFactoryTypes(set));

   HashMap map = new HashMap();
   map.put("javax.swing", Package.getPackage("javax.swing").getSpecificationVersion());
   RequiredPackages req = new RequiredPackages(map);
   retval.add(req);

   return(retval);
  }


  /**
   * A serializable factory for creating this UI
   */
  public static class Factory implements JComponentFactory, Serializable
  {


   /**
    * Returns a <CODE>JComponent</CODE>.
    */
   public JComponent getJComponent(Object roleObject)
   {
    ServiceItem item = (ServiceItem)roleObject;

    Logger.debug.println("DEBUG: [GnutellaSwingRepresentation2.factory] getJComponent called");
    try
      {
       GnutellaSwingRepresentation2 rep = new GnutellaSwingRepresentation2( ((GnutellaProxyIF)item.service) );
       return(rep.getPanel());
      }
    catch(RemoteException ex)
       {return(null);}

   }
  }


  public JPanel getPanel() throws RemoteException
  {
   return(this);
  }



  /**
   * Constructor
   */
 public GnutellaSwingRepresentation2(GnutellaProxyIF proxy)
 {
  try
    {
     Logger.debug.println("DEBUG: [GnutellaSwingRepresentation2] GnutellaSwingRepresentation2(...)");

     this.proxy = proxy;
     Lease l = proxy.addListener(new GnutellaSwingRepresentation2Listener(this), null, GnutellaEvent.EVID_FILEFOUND);
     if(l != null)
       {
        lrm.renewFor(l, Lease.FOREVER, null);
       };

     jbInit();

    }
  catch(Exception e)
    {
     Logger.err.println("Exception in GnutellaSwingRepresentation2::GnutellaSwingRepresentation2() has the Message: "+e.getMessage());
     e.printStackTrace(Logger.err);
    }
 }


  public void setFont(Font f)
  {
   if(f == null)
     return;
/*   if(jButton_go == null)
     return;*/

   super.setFont(f);
/*   jButton_back.setFont(f);
   jButton_next.setFont(f);
   jButton_go.setFont(f);
   jEditorPane_content.setFont(f);
   jComboBox_history.setFont(f);*/
  }

  /** Create UI.
    */
  private void jbInit() throws Exception
  {
    jPanel_connect = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    jButton_connect = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    jTextField_hostname = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());
    jTextField_port = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());
    jLabel1 = (JLabel) Beans.instantiate(getClass().getClassLoader(), JLabel.class.getName());
    jPanel_search = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    jButton_search = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    jTextField_searchstring = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());
    borderLayout1 = (BorderLayout) Beans.instantiate(getClass().getClassLoader(), BorderLayout.class.getName());
    jPanel_found = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    jList_found = (JList) Beans.instantiate(getClass().getClassLoader(), JList.class.getName());
    borderLayout2 = (BorderLayout) Beans.instantiate(getClass().getClassLoader(), BorderLayout.class.getName());
    jButton_download = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    jButton_connect.setText("connect");
    jButton_connect.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton_connect_actionPerformed(e);
      }
    });
    jTextField_hostname.setText("gnutellahosts.com");
    jTextField_port.setText("6346");
    jLabel1.setText(":");
    jButton_search.setText("add search");
    jButton_search.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton_search_actionPerformed(e);
      }
    });
    jTextField_searchstring.setText("something");
    this.setLayout(borderLayout1);
    jPanel_found.setLayout(borderLayout2);
    jButton_download.setEnabled(false);
    jButton_download.setText("download");
    jButton_download.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jButton_download_actionPerformed(e);
      }
    });
    this.add(jPanel_connect, BorderLayout.NORTH);
    jPanel_connect.add(jTextField_hostname, null);
    jPanel_connect.add(jLabel1, null);
    jPanel_connect.add(jTextField_port, null);
    jPanel_connect.add(jButton_connect, null);
    this.add(jPanel_search, BorderLayout.SOUTH);
    jPanel_search.add(jTextField_searchstring, null);
    jPanel_search.add(jButton_search, null);
    this.add(jPanel_found, BorderLayout.CENTER);
    jPanel_found.add(jList_found, BorderLayout.NORTH);
    jPanel_found.add(jButton_download, BorderLayout.SOUTH);
  }



  /** React to events from our service.
    */
  public void notify(RemoteEvent evt) throws UnknownEventException, RemoteException
  {

   if(!(evt instanceof GnutellaEvent))
     throw(new UnknownEventException("expected GnutellaEvent"));

   GnutellaEvent e = (GnutellaEvent)evt;

   if(debug)Logger.debug.println("DEBUG: [GnutellaSwingRepresentation2] notify type="+(int)e.getID()+" data=\""+e.data+"\"");

   switch((int)e.getID())
   {
    case GnutellaEvent.EVID_FILEFOUND:
      {
       ((DefaultListModel)jList_found.getModel()).addElement(e.data);
       jButton_download.setEnabled(true);
       break;
      }
    default: throw(new UnknownEventException("unknown event-id"));
   }
  }

  void jButton_connect_actionPerformed(ActionEvent evt)
  {
   try
     {
      proxy.connect(jTextField_hostname.getText(), Integer.parseInt(jTextField_port.getText()));
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }

  void jButton_search_actionPerformed(ActionEvent evt)
  {
   try
     {
      proxy.addSearch(jTextField_searchstring.getText(), 100, 0);
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }

  void jButton_download_actionPerformed(ActionEvent evt)
  {
   try
     {
      String url = jList_found.getSelectedValue().toString();
      if(url != null)
      if(url.length() > 0)
        proxy.download(url);
     }
   catch(Exception e)
     {
     }
  }

}